import warnings
from datetime import datetime
from pathlib import Path

from exiftool import ExifToolHelper

from PIL import ExifTags, Image, UnidentifiedImageError
from pillow_heif import register_heif_opener, register_avif_opener

register_heif_opener()
register_avif_opener()

def is_image(file: Path) -> bool:
    """
    Checks if a given file is an image.

    :param file: Path of the file to check.
    :returns: True if file is an image, False otherwise.
    """
    try:
        Image.Image.verify(file)
        return True
    except UnidentifiedImageError:
        return False

def has_date(file: Path) -> bool:
    """
    Checks if a given file has a date specified in the EXIF tags.

    :param file: The file to check.
    :returns: True if the file's EXIF tags contain a date, False otherwise.
    """

    return get_date(file, allow_none=True) is not None


def get_date(file: Path, allow_none: bool = False) -> str | None:
    """
    Fetch image capture date from EXIF metadata (or file modification timestamp).

    :param file: Path to image file
    :param allow_none: Return None for file without EXIF metadata.
    :returns: UNIX timestamp (from EXIF metadata or file modification) or None (if `allow_none` is specified)
    """
    try:
        with Image.open(file) as image:
            exif = image.getexif()
            ifd = exif.get_ifd(ExifTags.Base.ExifOffset)

            if 36867 in ifd:
                return datetime.strptime(ifd[36867], "%Y:%m:%d %H:%M:%S").timestamp()

            if 36868 in ifd:
                return datetime.strptime(ifd[36868], "%Y:%m:%d %H:%M:%S").timestamp()

            if 306 in exif:
                warnings.warn(
                    f"EXIF date not pulled from IFD from file {str(file)}. This is not supported by e.g. immich."
                )
                return datetime.strptime(exif[306], "%Y:%m:%d %H:%M:%S").timestamp()

            return None if allow_none else file.stat().st_ctime

    except UnidentifiedImageError:
        with ExifToolHelper() as helper:
            metadata = helper.get_metadata(file)
            return metadata.get("MediaCreateDate")


def set_date(file: Path, date: str) -> None:
    """
    Add EXIF date to a file.
    NOTE: This action irreversibly replaces the original image file.

    :param file: The path to an image file.
    :param date: The date to add as the creation date to `path`, formatted as "YYYY-MM-DD HH:MM:SS"
    """

    formatted_date = datetime.strptime(date, "%Y-%m-%d %H:%M:%S").strftime(
        "%Y:%m:%d %H:%M:%S"
    )

    try:
        with Image.open(file) as image:
            exif = image.getexif()
            ifd = exif.get_ifd(ExifTags.Base.ExifOffset)

            exif[306] = formatted_date
            ifd[36868] = formatted_date

            image.save(file, exif=exif)
    except UnidentifiedImageError:
        with ExifToolHelper() as helper:
            helper.execute(f"-MediaCreateDate={formatted_date}", file)
