from pathlib import Path
from uuid import uuid4
from typing import Callable

from imagetools.lib.utils import left_pad

def touch_file(file: Path) -> None:
    """
    Create a file.

    :param file: Path to the file that shall be created.
    """
    file.touch(exist_ok=True)


def append_to_file(file: Path, line: str) -> None:
    """
    Appends a given line to a given file.

    :param file: The path to the file that shall be appended to.
    :param line: The line to append to the File.
    """
    touch_file(file)

    with open(file, "a+") as f:
        f.write(line + "\n")
        f.close()


def scramble(directory: Path) -> None:
    """
    Rename all files in directory to random UUIDs.
    Helpful when trying to avoid naming collisions later on down the line.

    :param directory: The directory to use.
    """

    file_paths = [
        i
        for i in directory.iterdir()
        if i.is_file()
    ]

    for file in file_paths:
        extension = file.suffix
        new_file_path = file.parent.joinpath(f"{uuid4()}{extension}")
        file.rename(new_file_path)


def filter_ls(
    directory: Path,
    filter_method: Callable[[Path], bool] = lambda _: True,
    key_method: Callable[[Path], any] = lambda a: a,
) -> list:
    """
    Lists all files in a directory, then filters them based on a `filter_method`.
    Finally, the results are sorted based on a `sort_method`.

    :param directory: The directory to list
    :param filter_method: The method to call for filtering files.
    :param sort_method: The method to call for sorting files.
    :returns: Listing of all files within `directory`, filtered using `filter_method`.
    """
    file_paths = [
        i
        for i in directory.iterdir()
        if i.is_file()
    ]

    return sorted([i for i in file_paths if filter_method(i)], key=key_method)


def sequential_rename(paths: list[Path]) -> None:
    """
    Renames list of files based on their position in the list.

    :param paths: List of file paths to rename.
    """

    for i, file in enumerate(paths):
        extension = file.suffix
        new_file_path = file.parent.joinpath(f"{left_pad(str(i + 1), "0", 3)}{extension}")
        file.rename(new_file_path)
